<?php

namespace Drupal\ai_provider_xai\Plugin\AiProvider;

use GrokPHP\Client\Enums\Model;
use Symfony\Component\Yaml\Yaml;
use Drupal\ai\Attribute\AiProvider;
use GrokPHP\Client\Config\GrokConfig;
use GrokPHP\Client\Clients\GrokClient;
use GrokPHP\Client\Config\ChatOptions;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\ai\Base\AiProviderClientBase;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatOutput;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\OperationType\Chat\ChatInterface;
use Drupal\ai\Exception\AiResponseErrorException;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the xAI LLM.
 */
#[AiProvider(
  id: 'xai',
  label: new TranslatableMarkup('xAI'),
)]
class XAIProvider extends AiProviderClientBase implements ChatInterface {

  /**
   * The xAI client
   * 
   * @var GrokPHP\Client
   */
  protected $client;

  /**
   * API Key.
   *
   * @var string
   */
  protected string $apiKey = '';

  /**
   * {@inheritdoc}
   */
  public function getConfiguredModels(?string $operation_type = NULL, array $capabilities = []): array {
    return [
      'grok-2' => 'Grok 2',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isUsable(?string $operation_type = NULL, array $capabilities = []): bool {
    if (!$this->getConfig()->get('api_key')) {
      return FALSE;
    }

    if ($operation_type) {
      return in_array($operation_type, $this->getSupportedOperationTypes());
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedOperationTypes(): array {
    return [
      'chat',
    // 'embeddings',
    // 'moderation',
    // 'text_to_image',
    // 'text_to_speech',
    // 'speech_to_text',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(): ImmutableConfig {
    return $this->configFactory->get('ai_provider_xai.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getApiDefinition(): array {
    $definition = Yaml::parseFile(
      $this->moduleHandler->getModule('ai_provider_xai')
        ->getPath() . '/definitions/api_defaults.yml'
    );
    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public function getModelSettings(string $model_id, array $generalConfig = []): array {
    return $generalConfig;
  }

  /**
   * {@inheritdoc}
   */
  public function setAuthentication(mixed $authentication): void {
    $this->apiKey = $authentication;
    $this->client = NULL;
  }

  /**
   * Gets the raw client.
   *
   * @param string $api_key
   *   If the API key should be hot swapped.
   *
   * @return @todo add return type.
   *   The xAI client.
   */
  public function getClient(string $api_key = '') {
    if ($api_key) {
      $this->setAuthentication($api_key);
    }

    $this->loadClient();
    return $this->client;
  }

  /**
   * Loads the xAI Client with authentication if not initialized.
   */
  protected function loadClient(): void {
    if (!$this->client) {
      if (!$this->apiKey) {
        $this->setAuthentication($this->loadApiKey());
      }

      $config = new GrokConfig($this->apiKey);
      $this->client = new GrokClient($config);    
    }
  }

  /**
   * {@inheritdoc}
   */  
  public function chat(array|string|ChatInput $input, string $model_id, array $tags = []): ChatOutput {
    $this->loadClient();

    $messages = [
        // ['role' => 'system', 'content' => 'You are a helpful assistant.'].
        ['role' => 'user', 'content' => 'How do black holes form?'],
    ];

    // Custom API settings.
    $options = new ChatOptions(
        model: Model::GROK_2,
        temperature: 1.2,
        stream: false,
    );

    $response = $this->client->chat($messages, $options);
    $message = new ChatMessage('user', (string) $response['choices'][0]['message']['content']);

    return new ChatOutput($message, $response['choices'][0]['message']['content'], []);
  }

}
